<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\Category;
use SGalinski\SgNews\Domain\Model\News;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Controller that shows the latest news
 */
class LatestController extends AbstractController {
	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\TagRepository
	 */
	protected $tagRepository;

	/**
	 * @inject
	 * @var \SGalinski\SgNews\Domain\Repository\NewsRepository
	 */
	protected $newsRepository;

	/**
	 * Renders the news overview
	 *
	 * @param array $newsMetaData
	 * @param int $offset
	 * @return void
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 */
	public function indexAction(array $newsMetaData = [], $offset = 0) {
		$limit = ((int) $this->settings['limit']);
		$limit = ($limit < 1 ? 1 : $limit);

		$categoryUids = GeneralUtility::intExplode(',', $this->settings['categories'], TRUE);
		$tagUids = GeneralUtility::intExplode(',', $this->settings['tags'], TRUE);
		if ((int) $GLOBALS['TSFE']->page['doktype'] === 117 && !count($categoryUids)) {
			$categoryUids = [$GLOBALS['TSFE']->id];
		}

		$startTime = (int) $this->settings['starttime'];
		$endTime = (int) $this->settings['endtime'];
		$latestNewsEntries = $this->newsRepository->findLastUpdatedOrHighlightedNewsByCategories(
			$limit, FALSE, $categoryUids, $offset, TRUE, $this->settings['sortBy'], $tagUids, $startTime, $endTime
		);

		$categories = [];
		foreach ($latestNewsEntries as $latestNewsEntry) {
			/** @var News $latestNewsEntry */
			/** @var Category $category */
			$categoryUid = $latestNewsEntry->getPid();
			$category = $categories[$categoryUid];
			if (!$category) {
				$category = $this->categoryRepository->findByUid($categoryUid);
				if (!$category) {
					continue;
				}
				$categories[$categoryUid] = $category;
			}

			$newsMetaData[] = $this->getMetaDataForNews($latestNewsEntry, $category);
			// Make sure, that the amount is the same as the configured limit.
			if (count($newsMetaData) >= $limit) {
				break;
			}
		}

		// Redo this function, until $newsMetaData gets the $limit. Needed because of languagevisibility.
		if (count($newsMetaData) < $limit) {
			$offset += $limit - 1;
			$maxCount = $this->newsRepository->getCountOfLastUpdatedOrHighlightedNewsByCategories(
				FALSE, $categoryUids, TRUE, $this->settings['sortBy'], $tagUids, $startTime, $endTime
			);
			if ($offset < $maxCount) {
				$this->indexAction($newsMetaData, $offset);
				return;
			}
		}


		$this->view->assign('newsMetaData', $newsMetaData);
	}
}
